@@ -1,4 +1,3 @@
-#!/usr/bin/env ruby
 # -*- coding: utf-8 -*-
 require File.dirname(__FILE__) + '/test_helper'
 require 'sass/scss/css_parser'
@@ -49,26 +48,13 @@ baz {bar: baz}
 SCSS
   end
 
-  if Sass::Util.ruby1_8?
-    def test_unicode
-      assert_parses <<SCSS
+  def test_unicode
+    assert_parses <<SCSS
 @charset "UTF-8";
 foo {
   bar: föö bâr; }
 SCSS
-      assert_parses <<SCSS
-foo {
-  bar: föö bâr; }
-SCSS
-    end
-  else
-    def test_unicode
-      assert_parses <<SCSS
-@charset "UTF-8";
-foo {
-  bar: föö bâr; }
-SCSS
-      assert_equal <<CSS, render(<<SCSS)
+    assert_equal <<CSS, render(<<SCSS)
 @charset "UTF-8";
 foo {
   bar: föö bâr; }
@@ -76,7 +62,6 @@ CSS
 foo {
   bar: föö bâr; }
 SCSS
-    end
   end
 
   def test_invisible_comments
@@ -634,14 +619,14 @@ SCSS
 
   def test_supports
     assert_equal <<CSS, render(<<SCSS)
-@supports (a: b) and (c: d) or (not (d: e)) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
+@supports (((a: b) and (c: d)) or (not (d: e))) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
   .foo {
     a: b; } }
 @supports (a: b) {
   .foo {
     a: b; } }
 CSS
-@supports (a: b) and (c: d) or (not (d: e)) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
+@supports (((a: b) and (c: d)) or (not (d: e))) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
   .foo {
     a: b;
   }
@@ -653,13 +638,38 @@ CSS
   }
 }
 SCSS
+  end
 
+  def test_supports_with_prefix
     assert_equal <<CSS, render(<<SCSS)
-@-prefix-supports (a: b) and (c: d) or (not (d: e)) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
+@-prefix-supports (((a: b) and (c: d)) or (not (d: e))) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
   .foo {
     a: b; } }
 CSS
-@-prefix-supports (a: b) and (c: d) or (not (d: e)) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
+@-prefix-supports (((a: b) and (c: d)) or (not (d: e))) and ((not (f: g)) or (not ((h: i) and (j: k)))) {
+  .foo {
+    a: b;
+  }
+}
+SCSS
+  end
+
+  def test_supports_allows_similar_operators_without_parens
+    assert_equal <<CSS, render(<<SCSS)
+@supports (a: b) and (c: d) and (e: f) {
+  .foo {
+    a: b; } }
+@supports (a: b) or (c: d) or (e: f) {
+  .foo {
+    a: b; } }
+CSS
+@supports (a: b) and (c: d) and (e: f) {
+  .foo {
+    a: b;
+  }
+}
+
+@supports (a: b) or (c: d) or (e: f) {
   .foo {
     a: b;
   }
@@ -794,10 +804,10 @@ SCSS
     assert_selector_parses('E > F')
     assert_selector_parses('E + F')
     assert_selector_parses('E ~ F')
-    assert_selector_parses('E /foo/ F')
+    silence_warnings {assert_selector_parses('E /foo/ F')}
     silence_warnings {assert_selector_parses('E! > F')}
 
-    assert_selector_parses('E /ns|foo/ F')
+    silence_warnings {assert_selector_parses('E /ns|foo/ F')}
 
     # From http://dev.w3.org/csswg/css-scoping-1/
     assert_selector_parses('E:host(s)')
@@ -1013,6 +1023,12 @@ SCSS
     assert_equal "E + F {\n  a: b; }\n", render("E+F { a: b;} ")
   end
 
+  def test_escapes_in_selectors
+    assert_selector_parses('.\!foo')
+    assert_selector_parses('.\66 foo')
+    assert_selector_parses('.\21 foo')
+  end
+
   def test_subject_selector_deprecation
     assert_warning(<<WARNING) {render(".foo .bar! .baz {a: b}")}
 DEPRECATION WARNING on line 1, column 1:
@@ -1237,7 +1253,7 @@ SCSS
 
   def assert_directive_parses(param)
     assert_parses <<SCSS
-@keyframes #{param} {
+@unknown #{param} {
   a: b; }
 SCSS
   end
