@@ -1,4 +1,3 @@
-#!/usr/bin/env ruby
 # -*- coding: utf-8 -*-
 require File.dirname(__FILE__) + '/../test_helper'
 require File.dirname(__FILE__) + '/test_helper'
@@ -28,7 +27,7 @@ a {
 CSS
 {
 "version": 3,
-"mappings": "AAAA,CAAE;EACA,GAAG,EAAE,GAAG;;EAER,SAAS,EAAE,IAAI",
+"mappings": "AAAA,CAAE;EACA,GAAG,EAAE,GAAG;EACV,kBAAkB;EAChB,SAAS,EAAE,IAAI",
 "sources": ["test_simple_mapping_scss_inline.scss"],
 "names": [],
 "file": "test.css"
@@ -37,7 +36,7 @@ JSON
   end
 
   def test_simple_mapping_sass
-    assert_parses_with_sourcemap <<SASS, <<CSS, <<JSON, :syntax => :sass
+    silence_warnings {assert_parses_with_sourcemap <<SASS, <<CSS, <<JSON, :syntax => :sass}
 a
   foo: bar
   /* SOME COMMENT */
@@ -61,7 +60,7 @@ JSON
   end
 
   def test_simple_mapping_with_file_uris
-    uri = Sass::Util.file_uri_from_path(Sass::Util.absolute_path(filename_for_test(:scss)))
+    uri = Sass::Util.file_uri_from_path(File.absolute_path(filename_for_test(:scss)))
     assert_parses_with_sourcemap <<SCSS, <<CSS, <<JSON, :sourcemap => :file
 a {
   foo: bar;
@@ -78,7 +77,7 @@ a {
 CSS
 {
 "version": 3,
-"mappings": "AAAA,CAAE;EACA,GAAG,EAAE,GAAG;;EAER,SAAS,EAAE,IAAI",
+"mappings": "AAAA,CAAE;EACA,GAAG,EAAE,GAAG;EACV,kBAAkB;EAChB,SAAS,EAAE,IAAI",
 "sources": ["#{uri}"],
 "names": [],
 "file": "test.css"
@@ -104,7 +103,7 @@ a {
 CSS
 {
 "version": 3,
-"mappings": "AAAA,CAAE;EACA,GAAG,EAAE,GAAG;;EAER,SAAS,EAAE,IAAI",
+"mappings": "AAAA,CAAE;EACA,GAAG,EAAE,GAAG;EACV,kBAAkB;EAChB,SAAS,EAAE,IAAI",
 "sources": ["../scss/style.scss"],
 "names": [],
 "file": "style.css"
@@ -114,7 +113,7 @@ JSON
 
   def test_mapping_with_directory_sass
     options = {:filename => "sass/style.sass", :output => "css/style.css", :syntax => :sass}
-    assert_parses_with_sourcemap <<SASS, <<CSS, <<JSON, options
+    silence_warnings {assert_parses_with_sourcemap <<SASS, <<CSS, <<JSON, options}
 a
   foo: bar
   /* SOME COMMENT */
@@ -137,9 +136,8 @@ CSS
 JSON
   end
 
-  unless Sass::Util.ruby1_8?
-    def test_simple_charset_mapping_scss
-      assert_parses_with_sourcemap <<SCSS, <<CSS, <<JSON
+  def test_simple_charset_mapping_scss
+    assert_parses_with_sourcemap <<SCSS, <<CSS, <<JSON
 a {
   fóó: bár;
 }
@@ -158,10 +156,10 @@ CSS
 "file": "test.css"
 }
 JSON
-    end
+  end
 
-    def test_simple_charset_mapping_sass
-      assert_parses_with_sourcemap <<SASS, <<CSS, <<JSON, :syntax => :sass
+  def test_simple_charset_mapping_sass
+    assert_parses_with_sourcemap <<SASS, <<CSS, <<JSON, :syntax => :sass
 a
   fóó: bár
 SASS
@@ -179,10 +177,10 @@ CSS
 "file": "test.css"
 }
 JSON
-    end
+  end
 
-    def test_different_charset_than_encoding_scss
-      assert_parses_with_sourcemap(<<SCSS.force_encoding("IBM866"), <<CSS, <<JSON)
+  def test_different_charset_than_encoding_scss
+    assert_parses_with_sourcemap(<<SCSS.force_encoding("IBM866"), <<CSS, <<JSON)
 @charset "IBM866";
 f\x86\x86 {
   \x86: b;
@@ -202,10 +200,10 @@ CSS
 "file": "test.css"
 }
 JSON
-    end
+  end
 
-    def test_different_charset_than_encoding_sass
-      assert_parses_with_sourcemap(<<SASS.force_encoding("IBM866"), <<CSS, <<JSON, :syntax => :sass)
+  def test_different_charset_than_encoding_sass
+    assert_parses_with_sourcemap(<<SASS.force_encoding("IBM866"), <<CSS, <<JSON, :syntax => :sass)
 @charset "IBM866"
 f\x86\x86
   \x86: b
@@ -224,7 +222,6 @@ CSS
 "file": "test.css"
 }
 JSON
-    end
   end
 
   def test_import_sourcemap_scss
@@ -350,44 +347,42 @@ CSS
 $width: 2px;
 $translucent-red: rgba(255, 0, 0, 0.5);
 {{1}}a {{/1}}{
-  {{8}}.special {{/8}}{
-    {{9}}color{{/9}}: {{10}}red{{/10}};
-    {{11}}&:hover {{/11}}{
-      {{12}}foo{{/12}}: {{13}}bar{{/13}};
-      {{14}}cursor{{/14}}: {{15}}e + -resize{{/15}};
-      {{16}}color{{/16}}: {{17}}opacify($translucent-red, 0.3){{/17}};
+  {{9}}.special {{/9}}{
+    {{10}}color{{/10}}: {{11}}red{{/11}};
+    {{12}}&:hover {{/12}}{
+      {{13}}foo{{/13}}: {{14}}bar{{/14}};
+      {{15}}cursor{{/15}}: {{16}}e + -resize{{/16}};
+      {{17}}color{{/17}}: {{18}}opacify($translucent-red, 0.3){{/18}};
     }
-    {{18}}&:after {{/18}}{
-      {{19}}content{{/19}}: {{20}}"I ate #{5 + 10} pies #{$width} thick!"{{/20}};
+    {{19}}&:after {{/19}}{
+      {{20}}content{{/20}}: {{21}}"I ate #{5 + 10} pies #{$width} thick!"{{/21}};
     }
   }
-  {{21}}&:active {{/21}}{
-    {{22}}color{{/22}}: {{23}}#010203 + #040506{{/23}};
-    {{24}}border{{/24}}: {{25}}$width solid black{{/25}};
+  {{22}}&:active {{/22}}{
+    {{23}}border{{/23}}: {{24}}$width solid black{{/24}};
   }
-/* SOME COMMENT */
-  {{2}}font{{/2}}: {{3}}2px/3px {{/3}}{
-    {{4}}family{{/4}}: {{5}}fantasy{{/5}};
-    {{6}}size{{/6}}: {{7}}1em + (2em * 3){{/7}};
+{{2}}/* SOME COMMENT */{{/2}}
+  {{3}}font{{/3}}: {{4}}2px/3px {{/4}}{
+    {{5}}family{{/5}}: {{6}}fantasy{{/6}};
+    {{7}}size{{/7}}: {{8}}1em + (2em * 3){{/8}};
   }
 }
 SCSS
 {{1}}a{{/1}} {
-  /* SOME COMMENT */
-  {{2}}font{{/2}}: {{3}}2px/3px{{/3}};
-    {{4}}font-family{{/4}}: {{5}}fantasy{{/5}};
-    {{6}}font-size{{/6}}: {{7}}7em{{/7}}; }
-  {{8}}a .special{{/8}} {
-    {{9}}color{{/9}}: {{10}}red{{/10}}; }
-    {{11}}a .special:hover{{/11}} {
-      {{12}}foo{{/12}}: {{13}}bar{{/13}};
-      {{14}}cursor{{/14}}: {{15}}e-resize{{/15}};
-      {{16}}color{{/16}}: {{17}}rgba(255, 0, 0, 0.8){{/17}}; }
-    {{18}}a .special:after{{/18}} {
-      {{19}}content{{/19}}: {{20}}"I ate 15 pies 2px thick!"{{/20}}; }
-  {{21}}a:active{{/21}} {
-    {{22}}color{{/22}}: {{23}}#050709{{/23}};
-    {{24}}border{{/24}}: {{25}}2px solid black{{/25}}; }
+  {{2}}/* SOME COMMENT */{{/2}}
+  {{3}}font{{/3}}: {{4}}2px/3px{{/4}};
+    {{5}}font-family{{/5}}: {{6}}fantasy{{/6}};
+    {{7}}font-size{{/7}}: {{8}}7em{{/8}}; }
+  {{9}}a .special{{/9}} {
+    {{10}}color{{/10}}: {{11}}red{{/11}}; }
+    {{12}}a .special:hover{{/12}} {
+      {{13}}foo{{/13}}: {{14}}bar{{/14}};
+      {{15}}cursor{{/15}}: {{16}}e-resize{{/16}};
+      {{17}}color{{/17}}: {{18}}rgba(255, 0, 0, 0.8){{/18}}; }
+    {{19}}a .special:after{{/19}} {
+      {{20}}content{{/20}}: {{21}}"I ate 15 pies 2px thick!"{{/21}}; }
+  {{22}}a:active{{/22}} {
+    {{23}}border{{/23}}: {{24}}2px solid black{{/24}}; }
 
 /*# sourceMappingURL=test.css.map */
 CSS
@@ -398,39 +393,37 @@ CSS
 $width: 2px
 $translucent-red: rgba(255, 0, 0, 0.5)
 {{1}}a{{/1}}
-  {{8}}.special{{/8}}
-    {{9}}color{{/9}}: {{10}}red{{/10}}
-    {{11}}&:hover{{/11}}
-      {{12}}foo{{/12}}: {{13}}bar{{/13}}
-      {{14}}cursor{{/14}}: {{15}}e + -resize{{/15}}
-      {{16}}color{{/16}}: {{17}}opacify($translucent-red, 0.3){{/17}}
-    {{18}}&:after{{/18}}
-      {{19}}content{{/19}}: {{20}}"I ate #{5 + 10} pies #{$width} thick!"{{/20}}
-  {{21}}&:active{{/21}}
-    {{22}}color{{/22}}: {{23}}#010203 + #040506{{/23}}
-    {{24}}border{{/24}}: {{25}}$width solid black{{/25}}
+  {{9}}.special{{/9}}
+    {{10}}color{{/10}}: {{11}}red{{/11}}
+    {{12}}&:hover{{/12}}
+      {{13}}foo{{/13}}: {{14}}bar{{/14}}
+      {{15}}cursor{{/15}}: {{16}}e + -resize{{/16}}
+      {{17}}color{{/17}}: {{18}}opacify($translucent-red, 0.3){{/18}}
+    {{19}}&:after{{/19}}
+      {{20}}content{{/20}}: {{21}}"I ate #{5 + 10} pies #{$width} thick!"{{/21}}
+  {{22}}&:active{{/22}}
+    {{23}}border{{/23}}: {{24}}$width solid black{{/24}}
 
-  /* SOME COMMENT */
-  {{2}}font{{/2}}: {{3}}2px/3px{{/3}}
-    {{4}}family{{/4}}: {{5}}fantasy{{/5}}
-    {{6}}size{{/6}}: {{7}}1em + (2em * 3){{/7}}
+  {{2}}/* SOME COMMENT */{{/2}}
+  {{3}}font{{/3}}: {{4}}2px/3px{{/4}}
+    {{5}}family{{/5}}: {{6}}fantasy{{/6}}
+    {{7}}size{{/7}}: {{8}}1em + (2em * 3){{/8}}
 SASS
 {{1}}a{{/1}} {
-  /* SOME COMMENT */
-  {{2}}font{{/2}}: {{3}}2px/3px{{/3}};
-    {{4}}font-family{{/4}}: {{5}}fantasy{{/5}};
-    {{6}}font-size{{/6}}: {{7}}7em{{/7}}; }
-  {{8}}a .special{{/8}} {
-    {{9}}color{{/9}}: {{10}}red{{/10}}; }
-    {{11}}a .special:hover{{/11}} {
-      {{12}}foo{{/12}}: {{13}}bar{{/13}};
-      {{14}}cursor{{/14}}: {{15}}e-resize{{/15}};
-      {{16}}color{{/16}}: {{17}}rgba(255, 0, 0, 0.8){{/17}}; }
-    {{18}}a .special:after{{/18}} {
-      {{19}}content{{/19}}: {{20}}"I ate 15 pies 2px thick!"{{/20}}; }
-  {{21}}a:active{{/21}} {
-    {{22}}color{{/22}}: {{23}}#050709{{/23}};
-    {{24}}border{{/24}}: {{25}}2px solid black{{/25}}; }
+  {{2}}/* SOME COMMENT */{{/2}}
+  {{3}}font{{/3}}: {{4}}2px/3px{{/4}};
+    {{5}}font-family{{/5}}: {{6}}fantasy{{/6}};
+    {{7}}font-size{{/7}}: {{8}}7em{{/8}}; }
+  {{9}}a .special{{/9}} {
+    {{10}}color{{/10}}: {{11}}red{{/11}}; }
+    {{12}}a .special:hover{{/12}} {
+      {{13}}foo{{/13}}: {{14}}bar{{/14}};
+      {{15}}cursor{{/15}}: {{16}}e-resize{{/16}};
+      {{17}}color{{/17}}: {{18}}rgba(255, 0, 0, 0.8){{/18}}; }
+    {{19}}a .special:after{{/19}} {
+      {{20}}content{{/20}}: {{21}}"I ate 15 pies 2px thick!"{{/21}}; }
+  {{22}}a:active{{/22}} {
+    {{23}}border{{/23}}: {{24}}2px solid black{{/24}}; }
 
 /*# sourceMappingURL=test.css.map */
 CSS
@@ -667,7 +660,7 @@ CSS
   end
 
 def test_mixin_sourcemap_sass
-  assert_parses_with_mapping <<'SASS', <<'CSS', :syntax => :sass
+  silence_warnings {assert_parses_with_mapping <<'SASS', <<'CSS', :syntax => :sass}
 =large-text
   :font
     {{2}}size{{/2}}: {{3}}20px{{/3}}
@@ -761,7 +754,7 @@ CSS
   # Regression tests
 
   def test_properties_sass
-    assert_parses_with_mapping <<SASS, <<CSS, :syntax => :sass
+    silence_warnings {assert_parses_with_mapping <<SASS, <<CSS, :syntax => :sass}
 {{1}}.foo{{/1}}
   :{{2}}name{{/2}} {{3}}value{{/3}}
   {{4}}name{{/4}}: {{5}}value{{/5}}
@@ -800,13 +793,35 @@ SCSS
 
     interpolated = engine.to_tree.children.
       first.children.
-      first.value.children[1]
-    assert_equal interpolated.to_sass, "\#{123}"
+      first.value.first.children[1]
+    assert_equal "123", interpolated.to_sass
     range = interpolated.source_range
     assert_equal 3, range.start_pos.line
-    assert_equal 12, range.start_pos.offset
+    assert_equal 14, range.start_pos.offset
     assert_equal 3, range.end_pos.line
-    assert_equal 18, range.end_pos.offset
+    assert_equal 17, range.end_pos.offset
+  end
+
+  def test_list_source_range
+    engine = Sass::Engine.new(<<-SCSS, :cache => false, :syntax => :scss)
+@each $a, $b in (1, 2), (2, 4), (3, 6) { }
+SCSS
+    list = engine.to_tree.children.first.list
+    assert_equal 1, list.source_range.start_pos.line
+    assert_equal 1, list.source_range.end_pos.line
+    assert_equal 16, list.source_range.start_pos.offset
+    assert_equal 38, list.source_range.end_pos.offset
+  end
+
+  def test_map_source_range
+    engine = Sass::Engine.new(<<-SCSS, :cache => false, :syntax => :scss)
+$margins: (sm: 4px, md: 8px, lg: 16px);
+SCSS
+    expr = engine.to_tree.children.first.expr
+    assert_equal 1, expr.source_range.start_pos.line
+    assert_equal 1, expr.source_range.end_pos.line
+    assert_equal 12, expr.source_range.start_pos.offset
+    assert_equal 38, expr.source_range.end_pos.offset
   end
 
   def test_sources_array_is_uri_escaped
@@ -835,6 +850,72 @@ SCSS
 JSON
   end
 
+  def test_scss_comment_source_range
+    assert_parses_with_mapping <<SCSS, <<CSS, :syntax => :scss
+$var: val; {{1}}/* text */{{/1}}
+
+{{2}}/* multiline
+   comment */{{/2}}
+SCSS
+{{1}}/* text */{{/1}}
+{{2}}/* multiline
+   comment */{{/2}}
+
+/*# sourceMappingURL=test.css.map */
+CSS
+  end
+
+  def test_sass_comment_source_range
+    assert_parses_with_mapping <<SASS, <<CSS, :syntax => :sass
+{{1}}body{{/1}}
+  {{2}}/* text */{{/2}}
+
+{{3}}/* multiline
+   comment */{{/3}}
+SASS
+{{1}}body{{/1}} {
+  {{2}}/* text */{{/2}} }
+
+{{3}}/* multiline
+ * comment */{{/3}}
+
+/*# sourceMappingURL=test.css.map */
+CSS
+  end
+
+  def test_scss_comment_interpolation_source_range
+    assert_parses_with_mapping <<SCSS, <<CSS, :syntax => :scss
+$var: 2; {{1}}/* two \#{$var} and four \#{2 * $var} */{{/1}}
+
+{{2}}/* multiline
+   comment \#{ 2 + 2 } and \#{ 2 +
+   2 } */{{/2}}
+SCSS
+{{1}}/* two 2 and four 4 */{{/1}}
+{{2}}/* multiline
+   comment 4 and 4 */{{/2}}
+
+/*# sourceMappingURL=test.css.map */
+CSS
+  end
+
+  def test_sass_comment_interpolation_source_range
+    assert_parses_with_mapping <<SASS, <<CSS, :syntax => :sass
+$var: 2
+{{1}}/* two \#{$var} and four \#{2 * $var} */{{/1}}
+
+{{2}}/* multiline
+   comment \#{ 2 + 2 } and \#{ 2 +
+   2 } */{{/2}}
+SASS
+{{1}}/* two 2 and four 4 */{{/1}}
+{{2}}/* multiline
+ * comment 4 and 4 */{{/2}}
+
+/*# sourceMappingURL=test.css.map */
+CSS
+  end
+
   private
 
   ANNOTATION_REGEX = /\{\{(\/?)([^}]+)\}\}/
@@ -870,11 +951,12 @@ JSON
     source_ranges = build_ranges(source, source_file_name)
     target_ranges = build_ranges(css)
     map = Sass::Source::Map.new
-    Sass::Util.flatten(source_ranges.map do |(name, sources)|
+    source_ranges.map do |(name, sources)|
         assert(sources.length == 1, "#{sources.length} source ranges encountered for annotation #{name}")
         assert(target_ranges[name], "No target ranges for annotation #{name}")
         target_ranges[name].map {|target_range| [sources.first, target_range]}
-      end, 1).
+      end.
+      flatten(1).
       sort_by {|(_, target)| [target.start_pos.line, target.start_pos.offset]}.
       each {|(s2, target)| map.add(s2, target)}
     map
@@ -910,7 +992,11 @@ JSON
   def assert_ranges_equal(expected, actual, lines, prefix)
     assert_positions_equal(expected.start_pos, actual.start_pos, lines, prefix + " start position")
     assert_positions_equal(expected.end_pos, actual.end_pos, lines, prefix + " end position")
-    assert_equal(expected.file, actual.file)
+    if expected.file.nil?
+      assert_nil(actual.file)
+    else
+      assert_equal(expected.file, actual.file)
+    end
   end
 
   def assert_sourcemaps_equal(source, css, expected, actual)
